/*
        RADMON.C
        
        Steve Covert x7615
        (c) Jefferson Lab 2011
        Radmon firmware
*/
#include "uart.h"
#include "radmon.h"
#include "ls7366.h"
#include "mcp4921.h"
#include "generic_eeprom.h"


sbit ADC11_Direction at TRISB4_bit; // set up ADC on RB4


volatile sync_flag = 0;
volatile TMR0_flag = 0;
volatile int TMR0_counts;
int intValue;
unsigned long uLong;
unsigned long ls_arry[8];   // counts
unsigned long max_cps;      // max counts seen on any channel
int hivoltage;              // adc reading x 100 = hivoltage
float fTemp1;

unsigned int dac_temp;
float dFloat;
struct M sys;            // systam struct
char DAC_test_flag;
/* function prototypes */
void clear_alarms(void);



/* init for PIC18 series */
void init_main (void){
      ADCON1 |= 0x0F;      // Configure AN pins as digital
      CMCON  |= 7;         // Disable comparators
      INTCON2.B7 = 0;      // enable pullups on port b
      ADC11_Direction = 1; // input for ADC use
      
      TMR0_flag = 0;
      TMR0_counts = 0;

      // Timer0 Registers:// 16-Bit Mode; Prescaler=1:128; TMRH Preset=E1; TMRL Preset=7C; Freq=10.00064Hz; Period=99,993,600 ns
      T0CON.TMR0ON = 1;// Timer0 On/Off Control bit:1=Enables Timer0 / 0=Stops Timer0
      T0CON.T08BIT = 0;// Timer0 8-bit/16-bit Control bit: 1=8-bit timer/counter / 0=16-bit timer/counter
      T0CON.T0CS   = 0;// TMR0 Clock Source Select bit: 0=Internal Clock (CLKO) / 1=Transition on T0CKI pin
      T0CON.T0SE   = 0;// TMR0 Source Edge Select bit: 0=low/high / 1=high/low
      T0CON.PSA    = 0;// Prescaler Assignment bit: 0=Prescaler is assigned; 1=NOT assigned/bypassed
      T0CON.T0PS2  = 1;// bits 2-0  PS2:PS0: Prescaler Select bits
      T0CON.T0PS1  = 1;
      T0CON.T0PS0  = 0;
      TMR0H = 0xE1;    // preset for Timer0 MSB register
      TMR0L = 0x7C;    // preset for Timer0 LSB register

      TMR0IE_bit = 1;      // timer 0 interrupt enable
      TMR0IP_bit = 0;      // timer 0 lo priority

      /* UART */
      rs232_init();
      //counters
      ls_init();
      //dac
      DAC_Init();
      DAC_test_flag=0;
      
      /* Setup SPI bus */
      SPI1_Init_Advanced(_SPI_MASTER_OSC_DIV16, _SPI_DATA_SAMPLE_MIDDLE,
                   _SPI_CLK_IDLE_LOW, _SPI_LOW_2_HIGH);

      asm nop;
      clear_alarms();                 // flush alarms to 0
      max_cps = 0;
      
      /* check EEPROM & load system setup */
      // load whatever is in EEPROM to structure
      EEPROM_GET((char*)(&sys),sizeof(struct M),0);
      asm nop;
      if(sys.valid != 0xAA55) // test if valid
      {
          /* load some defaults */
          sys.valid = 0xAA55;         // used to verify EEPROM contents OK
          sys.address=99;
          sys.revision=VERSION;       // set revision
          sys.broadcast = 0;          // no broadcast
          sys.alarm[0].threshold = 99999;
          sys.alarm[1].threshold = 99999;
          sys.alarm[2].threshold = 99999;
          asm nop;
          // write eeprom
          EEPROM_PUT((char*)(&sys),sizeof(struct M),0);
          asm nop;
      }
      else
      {
          asm nop;
      }
      asm nop;
      asm nop;
      asm nop;

      // start up interrupts
      asm nop;
      IPEN_bit = 1;        // enable hi/lo prioritires
      RCIP_bit = 1;        // hi priority
      RCIE_bit = 1;        // rs232 interrupt en
      GIEL_bit = 1;        // all interrupts ON
      GIEH_bit = 1;        // all interrupts ON
      asm nop;

}


void interrupt_low(void)   //int_lo_priority() iv 0x0018
{
      // check timer0
      if (TMR0IF_bit)
      {
          TMR0IF_bit = 0;   // reset
          TMR0H = 0xE1;    // preset for Timer0 MSB register
          TMR0L = 0x7C;    // preset for Timer0 LSB register

          // 1 sec flag
          TMR0_counts = ++TMR0_counts % 10;  // one second
          if(TMR0_counts == 0)
          {
            TMR0_flag = 1;
          }
          asm nop;
          asm nop;
      }
}


void interrupt(void) //int_hi_priority() iv 0x0008
{
      if (RCIF_bit)
      {
              RCIF_bit = 0;            // clear int flag
              rxBuf[rxptr]=RCREG; // store byte in buf
              if(rxBuf[rxptr] == 13)
              {
                   rx_packet = 1;
                   asm nop;
                   asm nop;
              }
              rxptr = ++rxptr % 16; // inc ptr
              // we know no command is more than 14 bytes...
              if(rxptr > 14)  // overflow?
              {
                   rxptr = 0;       // reset pointer
                   asm nop;
              }
              asm nop;
      }
      asm nop;
      asm nop;
}

void clear_alarms(void)
{
     sys.alarm[0].set=0;
     sys.alarm[1].set=0;
     sys.alarm[2].set=0;
     asm nop;
}

void output_rev(void)
{
      PrintOut(PrintHandler, "%1.2f",sys.revision );
      PrintOut(PrintHandler, "\r");
}

// output counts for all channels
void output_counts(void)
{

      PrintOut(PrintHandler, "%5.0f,",(float)ls_arry[0]);
      PrintOut(PrintHandler, "%5.0f,",(float)ls_arry[1]);
      PrintOut(PrintHandler, "%5.0f,",(float)ls_arry[2]);
      PrintOut(PrintHandler, "%5.0f,",(float)ls_arry[3]);
      PrintOut(PrintHandler, "%5.0f,",(float)ls_arry[4]);
      PrintOut(PrintHandler, "%5.0f,",(float)ls_arry[5]);
      PrintOut(PrintHandler, "%5.0f,",(float)ls_arry[6]);
      PrintOut(PrintHandler, "%5.0f",(float)ls_arry[7]);
      PrintOut(PrintHandler, "\r");
}


/******************************************************/
void main()
{
  init_main();       // init all
  ADC_Init();        // init ADC
  
  
  asm nop;

  // take several readings on power up to get counters in sync
  for(intValue=0;intValue<3;intValue++)
  {
      while(TMR0_flag==0);
      TMR0_flag = 0;                 // reset flag
      ls_arry[0] = ls_counts(0);     // read counter
      ls_arry[1] = ls_counts(1);     // read counter
      ls_arry[2] = ls_counts(2);     // read counter
      ls_arry[3] = ls_counts(3);     // read counter
      ls_arry[4] = ls_counts(4);     // read counter
      ls_arry[5] = ls_counts(5);     // read counter
      ls_arry[6] = ls_counts(6);     // read counter
      ls_arry[7] = ls_counts(7);     // read counter
      ls_reset();                    // clear counters
      asm nop;
      asm nop;
  }

  while(1)
  {
       asm CLRWDT;    // asm line, clear WatchDog Timer

       // read coutners every 1.00000000 seconds
       // at same time get the max cps seen for this cycle of reads
       if(TMR0_flag)
       {
            TMR0_flag = 0;                 // reset flag
            ls_arry[0] = ls_counts(0);     // read counter
            max_cps = ls_arry[0];          // reset var

            ls_arry[1] = ls_counts(1);     // read counter
            if(ls_arry[1] > max_cps) max_cps = ls_arry[1];

            ls_arry[2] = ls_counts(2);     // read counter
            if(ls_arry[2] > max_cps) max_cps = ls_arry[2];

            ls_arry[3] = ls_counts(3);     // read counter
            if(ls_arry[3] > max_cps) max_cps = ls_arry[3];

            ls_arry[4] = ls_counts(4);     // read counter
            if(ls_arry[4] > max_cps) max_cps = ls_arry[4];

            ls_arry[5] = ls_counts(5);     // read counter
            if(ls_arry[5] > max_cps) max_cps = ls_arry[5];

            ls_arry[6] = ls_counts(6);     // read counter
            if(ls_arry[6] > max_cps) max_cps = ls_arry[6];

            ls_arry[7] = ls_counts(7);     // read counter
            if(ls_arry[7] > max_cps) max_cps = ls_arry[7];

            ls_reset();                    // clear counters
            asm nop;
            if(sys.broadcast)              // if broadcasting
            {
                 output_counts();          // do it
                 asm nop;
            }

             // check alarm values (RELAY0 normally OFF; relay1 normally ON)
             if(max_cps > sys.alarm[1].set) // norm open relay
             {
                  RA4_bit = 1;              // close relay
             }
             else
             {
                  RA4_bit = 0;              // open relay
             }

             if(max_cps > sys.alarm[2].set) // norm closed relay
             {
                  RA5_bit = 0;              // open relay
             }
             else
             {
                  RA5_bit = 1;              // close relay
             }

             // read ADC
             hivoltage = ADC_Read(11);
             asm nop;
             asm nop;
             asm nop;
             fTemp1 = (float)(hivoltage*2) * 0.488;
             hivoltage = (int)(fTemp1);
             asm nop;
             asm nop;
             asm nop;
       }


       if(rx_packet)                       // if we got A packet
       {
           decode_rs232();                 // decode it
           rx_packet = 0;
           asm nop;
       }

  } // end while

}//end main